@@ -12,6 +12,10 @@ def requires_prep(func):
     return wrapper
 
 
+class FilterException(Exception):
+    pass
+
+
 class MergeQuerySet(object):
     """
     This class provides a queryset-like object that merges two querysets
@@ -58,6 +62,12 @@ class MergeQuerySet(object):
         else:
             raise IndexError("list index out of range")
 
+    def filter(self, *args, **kwargs):
+        if not self.needs_prep:
+            raise FilterException("Filter called after queryset already merged")
+        return MergeQuerySet(self.queryset.filter(*args, **kwargs),
+                self.queryset2.filter(*args, **kwargs))
+
     @requires_prep
     def count(self):
         return self.__len__()
@@ -135,10 +145,18 @@ class GenericForeignKeyQuerySet(object):
     def __len__(self):
         return len(self.content)
 
+    def count(self):
+        return self.__len__()
+
     @requires_prep
     def __getitem__(self, i):
         return self.content.__getitem__(i)
 
+    def filter(self, *args, **kwargs):
+        if kwargs:
+            raise FilterException("GenericForeignKeyQuerySets cannot be filtered")
+        return self
+
     def __getattr__(self, key):
         try:
             return getattr(super(GenericForeignKeyQuerySet, self), key)
